#include <vector>
#include <algorithm>
#include <set>

#include<vcg/simplex/vertex/base.h>
#include<vcg/simplex/face/base.h>
#include<vcg/complex/trimesh/base.h>

#include<vcg/complex/trimesh/update/flag.h>
#include<vcg/simplex/face/pos.h>
#include<vcg/complex/trimesh/update/topology.h>

#include<wrap/io_trimesh/import_ply.h>
#include<wrap/io_trimesh/export_ply.h>

using namespace vcg;

class MyEdge;    // dummy prototype never used
class MyFace;
class MyVertex;

class MyVertex  : public VertexSimp2< MyVertex, MyEdge, MyFace, vertex::Coord3f, vertex::BitFlags, vertex::Color4b, vertex::VFAdj >{};
class MyFace    : public FaceSimp2  < MyVertex, MyEdge, MyFace, face::VertexRef, face::BitFlags, face::FFAdj, face::VFAdj > {};

class MyMesh : public tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace > >{};


int main(int argc,char **argv ){

	MyMesh m;
    if(argc==1) return 0;

	if(tri::io::ImporterPLY<MyMesh>::Open(m,argv[1])!=0)
		{
      printf("Error reading file  %s\n",argv[1]);
			exit(0);
		}
		
	printf("vn %i fn %i\n",m.vn, m.fn);
	printf("vn %i fn %i\n",m.vert.size(), m.face.size());

    tri::UpdateFlags<MyMesh>::VertexClearV(m);
    MyMesh::FaceIterator fi;
    for(fi=m.face.begin();fi!=m.face.end();++fi)
    {
        for(int i=0;i<3;++i)
            {
                (*fi).V(i)->SetV();
            }
    }

    MyMesh::VertexIterator vi;
    int unreferencedCount =0;
    for(vi=m.vert.begin();vi!=m.vert.end();++vi)
        if(!(*vi).IsV()) {
            unreferencedCount++;
            tri::Allocator<MyMesh>::DeleteVertex(m,*vi);
        }

    printf("Unreferenced vert:%i\n",unreferencedCount);

    printf("vn %i fn %i\n",m.vn, m.fn);
    printf("vn %i fn %i\n",m.vert.size(), m.face.size());

    tri::Allocator<MyMesh>::CompactVertexVector(m);
/*
    printf("vn %i fn %i\n",m.vn, m.fn);
    printf("vn %i fn %i\n",m.vert.size(), m.face.size());
    MyMesh::FacePointer myfp = &*m.face.begin();
    tri::Allocator<MyMesh>::PointerUpdater<MyFace *> pu;
    printf("Face base vector %u , %u \n",(int)(&*m.face.begin()),myfp);
    tri::Allocator<MyMesh>::AddFaces(m,1000,pu);
    printf("Face base vector %u , %u\n",(int)(&*m.face.begin()),myfp);
    pu.Update(myfp);
    printf("Face base vector %u , %u\n",(int)(&*m.face.begin()),myfp);
*/
    tri::UpdateTopology<MyMesh>::FaceFace(m);
    int borderEdgeCounter=0;
    int borderVertexCounter=0;

    tri::UpdateFlags<MyMesh>::VertexClearV(m);
    for(fi=m.face.begin();fi!=m.face.end();++fi)
    {
        for(int i=0;i<3;++i)
            {
                if((*fi).FFp(i) == &*fi)
                    {
                        borderEdgeCounter++;
                        (*fi).V0(i)->SetV();
                        (*fi).V1(i)->SetV();
                    }
            }
    }

    for(vi=m.vert.begin();vi!=m.vert.end();++vi)
        if((*vi).IsV()) {
            borderVertexCounter++;
            (*vi).C()=Color4b::Red;
        }

    printf("border vert:%i\n",borderVertexCounter);

    printf("Mesh has %i border edge\n",borderEdgeCounter);

    tri::io::ExporterPLY<MyMesh>::Save(m,"pippo.ply",tri::io::Mask::IOM_ALL);

    tri::UpdateTopology<MyMesh>::VertexFace(m);

     for(vi=m.vert.begin();vi!=m.vert.end();++vi)
        {
         int valCnt=0;
         for(face::VFIterator<MyFace> vfi(&*vi); !vfi.End(); ++vfi)
             valCnt++;

         if(valCnt==6)  (*vi).C()=Color4b::Blue;
         else  (*vi).C()=Color4b::Green;
        }
    tri::io::ExporterPLY<MyMesh>::Save(m,"pippo2.ply",tri::io::Mask::IOM_ALL);

  // add a per-vertex attribute with type float named "Irradiance"
  MyMesh::PerVertexAttributeHandle<int> ih = vcg::tri::Allocator<MyMesh>::AddPerVertexAttribute<int>  (m,std::string("Valency"));

  for(vi=m.vert.begin();vi!=m.vert.end();++vi)
        {
            ih[vi] = 0;
        }

  return 0;
}

